/*!
 * @nuxt/cli v2.8.1 (c) 2016-2019

 * - All the amazing contributors
 * Released under the MIT License.
 * Website: https://nuxtjs.org
*/
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

const __chunk_1 = require('./cli-chunk.js');
require('path');
const consola = _interopDefault(require('consola'));
require('minimist');
require('exit');
require('@nuxt/utils');
const chalk = _interopDefault(require('chalk'));
require('std-env');
require('wrap-ansi');
require('boxen');
const __chunk_2 = require('./cli-chunk2.js');
require('@nuxt/config');
require('esm');
require('fs-extra');
const __chunk_3 = require('./cli-chunk3.js');

async function listCommands() {
  const commandsOrder = ['dev', 'build', 'generate', 'start', 'help'];

  // Load all commands
  const _commands = await Promise.all(
    commandsOrder.map(cmd => __chunk_1.getCommand(cmd))
  );

  let maxLength = 0;
  const commandsHelp = [];

  for (const command of _commands) {
    commandsHelp.push([command.usage, command.description]);
    maxLength = Math.max(maxLength, command.usage.length);
  }

  const _cmds = commandsHelp.map(([cmd, description]) => {
    const i = __chunk_2.indent(maxLength + __chunk_2.optionSpaces - cmd.length);
    return __chunk_2.foldLines(
      chalk.green(cmd) + i + description,
      __chunk_2.startSpaces + maxLength + __chunk_2.optionSpaces * 2,
      __chunk_2.startSpaces + __chunk_2.optionSpaces
    )
  }).join('\n');

  const usage = __chunk_2.foldLines(`Usage: nuxt <command> [--help|-h]`, __chunk_2.startSpaces);
  const cmds = __chunk_2.foldLines(`Commands:`, __chunk_2.startSpaces) + '\n\n' + _cmds;

  process.stderr.write(__chunk_2.colorize(`${usage}\n\n${cmds}\n\n`));
}

const help = {
  name: 'help',
  description: 'Shows help for <command>',
  usage: 'help <command>',
  options: {
    help: __chunk_3.common.help,
    version: __chunk_3.common.version
  },
  async run(cmd) {
    const [name] = cmd._argv;
    if (!name) {
      return listCommands()
    }
    const command = await __chunk_1.getCommand(name);

    if (!command) {
      consola.info(`Unknown command: ${name}`);
      return
    }

    __chunk_1.NuxtCommand.from(command).showHelp();
  }
};

exports.default = help;
